@@ -18,6 +18,9 @@
 
 #include "../Audacity.h"
 #include "Distortion.h"
+#include "LoadEffects.h"
+
+#include "../Experimental.h"
 
 #include <cmath>
 #include <algorithm>
@@ -31,15 +34,18 @@
 #define M_PI_2 1.57079632679489661923132169163975 
 #endif
 
+#include <wx/checkbox.h>
 #include <wx/choice.h>
 #include <wx/intl.h>
 #include <wx/valgen.h>
 #include <wx/log.h>
+#include <wx/slider.h>
+#include <wx/stattext.h>
 
 #include "../Prefs.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../widgets/valnum.h"
-#include "../TranslatableStringArray.h"
 
 enum kTableType
 {
@@ -54,29 +60,29 @@ enum kTableType
    kLeveller,
    kRectifier,
    kHardLimiter,
-   kNumTableTypes
+   nTableTypes
 };
 
-static const wxString kTableTypeStrings[kNumTableTypes] =
+static const EnumValueSymbol kTableTypeStrings[nTableTypes] =
 {
-   XO("Hard Clipping"),
-   XO("Soft Clipping"),
-   XO("Soft Overdrive"),
-   XO("Medium Overdrive"),
-   XO("Hard Overdrive"),
-   XO("Cubic Curve (odd harmonics)"),
-   XO("Even Harmonics"),
-   XO("Expand and Compress"),
-   XO("Leveller"),
-   XO("Rectifier Distortion"),
-   XO("Hard Limiter 1413")
+   { XO("Hard Clipping") },
+   { XO("Soft Clipping") },
+   { XO("Soft Overdrive") },
+   { XO("Medium Overdrive") },
+   { XO("Hard Overdrive") },
+   { XO("Cubic Curve (odd harmonics)") },
+   { XO("Even Harmonics") },
+   { XO("Expand and Compress") },
+   { XO("Leveller") },
+   { XO("Rectifier Distortion") },
+   { XO("Hard Limiter 1413") }
 };
 
 // Define keys, defaults, minimums, and maximums for the effect parameters
 // (Note: 'Repeats' is the total number of times the effect is applied.)
 //
 //     Name             Type     Key                   Def     Min      Max                 Scale
-Param( TableTypeIndx,   int,     wxT("Type"),           0,       0,      kNumTableTypes-1,    1    );
+Param( TableTypeIndx,   int,     wxT("Type"),           0,       0,      nTableTypes-1,    1    );
 Param( DCBlock,         bool,    wxT("DC Block"),      false,   false,   true,                1    );
 Param( Threshold_dB,    double,  wxT("Threshold dB"),  -6.0,  -100.0,     0.0,             1000.0f );
 Param( NoiseFloor,      double,  wxT("Noise Floor"),   -70.0,  -80.0,   -20.0,                1    );
@@ -91,7 +97,7 @@ const double MIN_Threshold_Linear DB_TO_LINEAR(MIN_Threshold_dB);
 
 static const struct
 {
-   const wxChar *name;
+   const TranslatableString name;
    EffectDistortion::Params params;
 }
 FactoryPresets[] =
@@ -123,9 +129,9 @@ FactoryPresets[] =
    { XO("Percussion Limiter"),                    {10,        0,      -12.0,      -70.0,     100.0,    30.0,    0 } },
 };
 
-wxString defaultLabel(int index)
+TranslatableString defaultLabel(int index)
 {
-   static const wxString names[] = {
+   static const TranslatableString names[] = {
       XO("Upper Threshold"),
       XO("Noise Floor"),
       XO("Parameter 1"),
@@ -133,27 +139,18 @@ wxString defaultLabel(int index)
       XO("Number of repeats"),
    };
 
-   class NamesArray final : public TranslatableStringArray
-   {
-      void Populate() override
-      {
-         for (auto &name : names)
-            mContents.push_back( wxGetTranslation( name ) );
-      }
-   };
-
-   static NamesArray theArray;
-
-   return theArray.Get()[ index ];
+   return names[ index ];
 }
 
-#include <wx/arrimpl.cpp>
-WX_DEFINE_OBJARRAY(EffectDistortionStateArray);
- 
 //
 // EffectDistortion
 //
 
+const ComponentInterfaceSymbol EffectDistortion::Symbol
+{ XO("Distortion") };
+
+namespace{ BuiltinEffectsModule::Registration< EffectDistortion > reg; }
+
 BEGIN_EVENT_TABLE(EffectDistortion, wxEvtHandler)
    EVT_CHOICE(ID_Type, EffectDistortion::OnTypeChoice)
    EVT_CHECKBOX(ID_DCBlock, EffectDistortion::OnDCBlockCheckbox)
@@ -171,7 +168,7 @@ END_EVENT_TABLE()
 
 EffectDistortion::EffectDistortion()
 {
-   wxASSERT(kNumTableTypes == WXSIZEOF(kTableTypeStrings));
+   wxASSERT(nTableTypes == WXSIZEOF(kTableTypeStrings));
 
    mParams.mTableChoiceIndx = DEF_TableTypeIndx;
    mParams.mDCBlock = DEF_DCBlock;
@@ -191,16 +188,16 @@ EffectDistortion::~EffectDistortion()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectDistortion::GetSymbol()
+ComponentInterfaceSymbol EffectDistortion::GetSymbol()
 {
-   return DISTORTION_PLUGIN_SYMBOL;
+   return Symbol;
 }
 
-wxString EffectDistortion::GetDescription()
+TranslatableString EffectDistortion::GetDescription()
 {
-   return _("Waveshaping distortion effect");
+   return XO("Waveshaping distortion effect");
 }
 
 wxString EffectDistortion::ManualPage()
@@ -208,7 +205,7 @@ wxString EffectDistortion::ManualPage()
    return wxT("Distortion");
 }
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
 EffectType EffectDistortion::GetType()
 {
@@ -251,7 +248,7 @@ bool EffectDistortion::RealtimeInitialize()
 {
    SetBlockSize(512);
 
-   mSlaves.Clear();
+   mSlaves.clear();
 
    return true;
 }
@@ -262,14 +259,14 @@ bool EffectDistortion::RealtimeAddProcessor(unsigned WXUNUSED(numChannels), floa
 
    InstanceInit(slave, sampleRate);
 
-   mSlaves.Add(slave);
+   mSlaves.push_back(slave);
 
    return true;
 }
 
 bool EffectDistortion::RealtimeFinalize()
 {
-   mSlaves.Clear();
+   mSlaves.clear();
 
    return true;
 }
@@ -282,10 +279,22 @@ size_t EffectDistortion::RealtimeProcess(int group,
 
    return InstanceProcess(mSlaves[group], inbuf, outbuf, numSamples);
 }
+bool EffectDistortion::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_ENUM_PARAM( mParams.mTableChoiceIndx, TableTypeIndx,
+      kTableTypeStrings, nTableTypes );
+   S.SHUTTLE_PARAM( mParams.mDCBlock,       DCBlock       );
+   S.SHUTTLE_PARAM( mParams.mThreshold_dB,  Threshold_dB  );
+   S.SHUTTLE_PARAM( mParams.mNoiseFloor,    NoiseFloor    );
+   S.SHUTTLE_PARAM( mParams.mParam1,        Param1        );
+   S.SHUTTLE_PARAM( mParams.mParam2,        Param2        );
+   S.SHUTTLE_PARAM( mParams.mRepeats,       Repeats       );
+   return true;
+}
 
-bool EffectDistortion::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectDistortion::GetAutomationParameters(CommandParameters & parms)
 {
-   parms.Write(KEY_TableTypeIndx, kTableTypeStrings[mParams.mTableChoiceIndx]);
+   parms.Write(KEY_TableTypeIndx,
+               kTableTypeStrings[mParams.mTableChoiceIndx].Internal());
    parms.Write(KEY_DCBlock, mParams.mDCBlock);
    parms.Write(KEY_Threshold_dB, mParams.mThreshold_dB);
    parms.Write(KEY_NoiseFloor, mParams.mNoiseFloor);
@@ -296,9 +305,9 @@ bool EffectDistortion::GetAutomationParameters(EffectAutomationParameters & parm
    return true;
 }
 
-bool EffectDistortion::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectDistortion::SetAutomationParameters(CommandParameters & parms)
 {
-   ReadAndVerifyEnum(TableTypeIndx,  wxArrayString(kNumTableTypes, kTableTypeStrings));
+   ReadAndVerifyEnum(TableTypeIndx, kTableTypeStrings, nTableTypes);
    ReadAndVerifyBool(DCBlock);
    ReadAndVerifyDouble(Threshold_dB);
    ReadAndVerifyDouble(NoiseFloor);
@@ -317,13 +326,13 @@ bool EffectDistortion::SetAutomationParameters(EffectAutomationParameters & parm
    return true;
 }
 
-wxArrayString EffectDistortion::GetFactoryPresets()
+RegistryPaths EffectDistortion::GetFactoryPresets()
 {
-   wxArrayString names;
+   RegistryPaths names;
 
    for (size_t i = 0; i < WXSIZEOF(FactoryPresets); i++)
    {
-      names.Add(wxGetTranslation(FactoryPresets[i].name));
+      names.push_back( FactoryPresets[i].name.Translation() );
    }
 
    return names;
@@ -352,28 +361,25 @@ bool EffectDistortion::LoadFactoryPreset(int id)
 
 void EffectDistortion::PopulateOrExchange(ShuttleGui & S)
 {
-   for (int i = 0; i < kNumTableTypes; i++)
-   {
-      mTableTypes.Add(wxGetTranslation(kTableTypeStrings[i]));
-   }
-
    S.AddSpace(0, 5);
    S.StartVerticalLay();
    {
       S.StartMultiColumn(4, wxCENTER);
       {
-         mTypeChoiceCtrl = S.Id(ID_Type).AddChoice(_("Distortion type:"), wxT(""), &mTableTypes);
-         mTypeChoiceCtrl->SetValidator(wxGenericValidator(&mParams.mTableChoiceIndx));
-         S.SetSizeHints(-1, -1);
+         mTypeChoiceCtrl = S.Id(ID_Type)
+            .MinSize( { -1, -1 } )
+            .Validator<wxGenericValidator>(&mParams.mTableChoiceIndx)
+            .AddChoice(XO("Distortion type:"),
+               Msgids(kTableTypeStrings, nTableTypes));
 
-         mDCBlockCheckBox = S.Id(ID_DCBlock).AddCheckBox(_("DC blocking filter"),
-                                       DEF_DCBlock ? wxT("true") : wxT("false"));
+         mDCBlockCheckBox = S.Id(ID_DCBlock).AddCheckBox(XO("DC blocking filter"),
+                                       DEF_DCBlock);
       }
       S.EndMultiColumn();
       S.AddSpace(0, 10);
 
 
-      S.StartStatic(_("Threshold controls"));
+      S.StartStatic(XO("Threshold controls"));
       {
          S.StartMultiColumn(4, wxEXPAND);
          S.SetStretchyCol(2);
@@ -382,38 +388,45 @@ void EffectDistortion::PopulateOrExchange(ShuttleGui & S)
             S.AddSpace(250,0); S.AddSpace(0,0); S.AddSpace(0,0); S.AddSpace(0,0); 
 
             // Upper threshold control
-            mThresholdTxt = S.AddVariableText(defaultLabel(0), false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
-            FloatingPointValidator<double> vldThreshold(2, &mParams.mThreshold_dB);
-            vldThreshold.SetRange(MIN_Threshold_dB, MAX_Threshold_dB);
-            mThresholdT = S.Id(ID_Threshold).AddTextBox( {}, wxT(""), 10);
-            mThresholdT->SetName(defaultLabel(0));
-            mThresholdT->SetValidator(vldThreshold);
+            mThresholdTxt = S.AddVariableText(defaultLabel(0),
+               false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
+            mThresholdT = S.Id(ID_Threshold)
+               .Name(defaultLabel(0))
+               .Validator<FloatingPointValidator<double>>(
+                  2, &mParams.mThreshold_dB, NumValidatorStyle::DEFAULT,
+                  MIN_Threshold_dB, MAX_Threshold_dB)
+               .AddTextBox( {}, wxT(""), 10);
 
-            S.SetStyle(wxSL_HORIZONTAL);
-            double maxLin = DB_TO_LINEAR(MAX_Threshold_dB) * SCL_Threshold_dB;
-            double minLin = DB_TO_LINEAR(MIN_Threshold_dB) * SCL_Threshold_dB;
-            mThresholdS = S.Id(ID_Threshold).AddSlider( {}, 0, maxLin, minLin);
-            mThresholdS->SetName(defaultLabel(0));
+            mThresholdS = S.Id(ID_Threshold)
+               .Name(defaultLabel(0))
+               .Style(wxSL_HORIZONTAL)
+               .AddSlider( {}, 0,
+                  DB_TO_LINEAR(MAX_Threshold_dB) * SCL_Threshold_dB,
+                  DB_TO_LINEAR(MIN_Threshold_dB) * SCL_Threshold_dB);
             S.AddSpace(20, 0);
 
             // Noise floor control
-            mNoiseFloorTxt = S.AddVariableText(defaultLabel(1), false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
-            FloatingPointValidator<double> vldfloor(2, &mParams.mNoiseFloor);
-            vldfloor.SetRange(MIN_NoiseFloor, MAX_NoiseFloor);
-            mNoiseFloorT = S.Id(ID_NoiseFloor).AddTextBox( {}, wxT(""), 10);
-            mNoiseFloorT->SetName(defaultLabel(1));
-            mNoiseFloorT->SetValidator(vldfloor);
+            mNoiseFloorTxt = S.AddVariableText(defaultLabel(1),
+               false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
+            mNoiseFloorT = S.Id(ID_NoiseFloor)
+               .Name(defaultLabel(1))
+               .Validator<FloatingPointValidator<double>>(
+                  2, &mParams.mNoiseFloor, NumValidatorStyle::DEFAULT,
+                  MIN_NoiseFloor, MAX_NoiseFloor
+               )
+               .AddTextBox( {}, wxT(""), 10);
 
-            S.SetStyle(wxSL_HORIZONTAL);
-            mNoiseFloorS = S.Id(ID_NoiseFloor).AddSlider( {}, 0, MAX_NoiseFloor, MIN_NoiseFloor);
-            mNoiseFloorS->SetName(defaultLabel(1));
+            mNoiseFloorS = S.Id(ID_NoiseFloor)
+               .Name(defaultLabel(1))
+               .Style(wxSL_HORIZONTAL)
+               .AddSlider( {}, 0, MAX_NoiseFloor, MIN_NoiseFloor);
             S.AddSpace(20, 0);
          }
          S.EndMultiColumn();
       }
       S.EndStatic();
 
-      S.StartStatic(_("Parameter controls"));
+      S.StartStatic(XO("Parameter controls"));
       {
          S.StartMultiColumn(4, wxEXPAND);
          S.SetStretchyCol(2);
@@ -422,42 +435,54 @@ void EffectDistortion::PopulateOrExchange(ShuttleGui & S)
             S.AddSpace(250,0); S.AddSpace(0,0); S.AddSpace(0,0); S.AddSpace(0,0); 
 
             // Parameter1 control
-            mParam1Txt = S.AddVariableText(defaultLabel(2), false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
-            FloatingPointValidator<double> vldparam1(2, &mParams.mParam1);
-            vldparam1.SetRange(MIN_Param1, MAX_Param1);
-            mParam1T = S.Id(ID_Param1).AddTextBox( {}, wxT(""), 10);
-            mParam1T->SetName(defaultLabel(2));
-            mParam1T->SetValidator(vldparam1);
+            mParam1Txt = S.AddVariableText(defaultLabel(2),
+               false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
+            mParam1T = S.Id(ID_Param1)
+               .Name(defaultLabel(2))
+               .Validator<FloatingPointValidator<double>>(
+                  2, &mParams.mParam1, NumValidatorStyle::DEFAULT,
+                  MIN_Param1, MAX_Param1
+               )
+               .AddTextBox( {}, wxT(""), 10);
 
-            S.SetStyle(wxSL_HORIZONTAL);
-            mParam1S = S.Id(ID_Param1).AddSlider( {}, 0, MAX_Param1, MIN_Param1);
-            mParam1S->SetName(defaultLabel(2));
+            mParam1S = S.Id(ID_Param1)
+               .Name(defaultLabel(2))
+               .Style(wxSL_HORIZONTAL)
+               .AddSlider( {}, 0, MAX_Param1, MIN_Param1);
             S.AddSpace(20, 0);
 
             // Parameter2 control
-            mParam2Txt = S.AddVariableText(defaultLabel(3), false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
-            FloatingPointValidator<double> vldParam2(2, &mParams.mParam2);
-            vldParam2.SetRange(MIN_Param2, MAX_Param2);
-            mParam2T = S.Id(ID_Param2).AddTextBox( {}, wxT(""), 10);
-            mParam2T->SetName(defaultLabel(3));
-            mParam2T->SetValidator(vldParam2);
+            mParam2Txt = S.AddVariableText(defaultLabel(3),
+               false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
+            mParam2T = S.Id(ID_Param2)
+               .Name(defaultLabel(3))
+               .Validator<FloatingPointValidator<double>>(
+                  2, &mParams.mParam2, NumValidatorStyle::DEFAULT,
+                  MIN_Param2, MAX_Param2
+               )
+               .AddTextBox( {}, wxT(""), 10);
 
-            S.SetStyle(wxSL_HORIZONTAL);
-            mParam2S = S.Id(ID_Param2).AddSlider( {}, 0, MAX_Param2, MIN_Param2);
-            mParam2S->SetName(defaultLabel(3));
+            mParam2S = S.Id(ID_Param2)
+               .Name(defaultLabel(3))
+               .Style(wxSL_HORIZONTAL)
+               .AddSlider( {}, 0, MAX_Param2, MIN_Param2);
             S.AddSpace(20, 0);
 
             // Repeats control
-            mRepeatsTxt = S.AddVariableText(defaultLabel(4), false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
-            IntegerValidator<int>vldRepeats(&mParams.mRepeats);
-            vldRepeats.SetRange(MIN_Repeats, MAX_Repeats);
-            mRepeatsT = S.Id(ID_Repeats).AddTextBox( {}, wxT(""), 10);
-            mRepeatsT->SetName(defaultLabel(4));
-            mRepeatsT->SetValidator(vldRepeats);
+            mRepeatsTxt = S.AddVariableText(defaultLabel(4),
+               false, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
+            mRepeatsT = S.Id(ID_Repeats)
+               .Name(defaultLabel(4))
+               .Validator<IntegerValidator<int>>(
+                  &mParams.mRepeats, NumValidatorStyle::DEFAULT,
+                  MIN_Repeats, MAX_Repeats
+               )
+               .AddTextBox( {}, wxT(""), 10);
 
-            S.SetStyle(wxSL_HORIZONTAL);
-            mRepeatsS = S.Id(ID_Repeats).AddSlider( {}, DEF_Repeats, MAX_Repeats, MIN_Repeats);
-            mRepeatsS->SetName(defaultLabel(4));
+            mRepeatsS = S.Id(ID_Repeats)
+               .Name(defaultLabel(4))
+               .Style(wxSL_HORIZONTAL)
+               .AddSlider( {}, DEF_Repeats, MAX_Repeats, MIN_Repeats);
             S.AddSpace(20, 0);
          }
          S.EndMultiColumn();
@@ -692,12 +717,12 @@ void EffectDistortion::UpdateUI()
          UpdateControlText(mParam2T, mOldParam2Txt, true);
          UpdateControlText(mRepeatsT, mOldRepeatsTxt, false);
 
-         UpdateControl(ID_Threshold, true, _("Clipping level"));
+         UpdateControl(ID_Threshold, true, XO("Clipping level"));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Drive"));
-         UpdateControl(ID_Param2, true, _("Make-up Gain"));
+         UpdateControl(ID_Param1, true, XO("Drive"));
+         UpdateControl(ID_Param2, true, XO("Make-up Gain"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kSoftClip:
          UpdateControlText(mThresholdT, mOldThresholdTxt, true);
@@ -706,12 +731,12 @@ void EffectDistortion::UpdateUI()
          UpdateControlText(mParam2T, mOldParam2Txt, true);
          UpdateControlText(mRepeatsT, mOldRepeatsTxt, false);
 
-         UpdateControl(ID_Threshold, true, _("Clipping threshold"));
+         UpdateControl(ID_Threshold, true, XO("Clipping threshold"));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Hardness"));
-         UpdateControl(ID_Param2, true, _("Make-up Gain"));
+         UpdateControl(ID_Param1, true, XO("Hardness"));
+         UpdateControl(ID_Param2, true, XO("Make-up Gain"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kHalfSinCurve:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -722,10 +747,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
-         UpdateControl(ID_Param2, true, _("Output level"));
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
+         UpdateControl(ID_Param2, true, XO("Output level"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kExpCurve:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -736,10 +761,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
-         UpdateControl(ID_Param2, true, _("Output level"));
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
+         UpdateControl(ID_Param2, true, XO("Output level"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kLogCurve:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -750,10 +775,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
-         UpdateControl(ID_Param2, true, _("Output level"));
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
+         UpdateControl(ID_Param2, true, XO("Output level"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kCubic:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -764,10 +789,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
-         UpdateControl(ID_Param2, true, _("Output level"));
-         UpdateControl(ID_Repeats, true, _("Repeat processing"));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
+         UpdateControl(ID_Param2, true, XO("Output level"));
+         UpdateControl(ID_Repeats, true, XO("Repeat processing"));
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kEvenHarmonics:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -778,10 +803,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
-         UpdateControl(ID_Param2, true, _("Harmonic brightness"));
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
+         UpdateControl(ID_Param2, true, XO("Harmonic brightness"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, true, wxEmptyString);
+         UpdateControl(ID_DCBlock, true, {});
          break;
       case kSinCurve:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -792,10 +817,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
-         UpdateControl(ID_Param2, true, _("Output level"));
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
+         UpdateControl(ID_Param2, true, XO("Output level"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kLeveller:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -806,10 +831,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, true, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Levelling fine adjustment"));
+         UpdateControl(ID_Param1, true, XO("Levelling fine adjustment"));
          UpdateControl(ID_Param2, false, defaultLabel(3));
-         UpdateControl(ID_Repeats, true, _("Degree of Levelling"));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_Repeats, true, XO("Degree of Levelling"));
+         UpdateControl(ID_DCBlock, false, {});
          break;
       case kRectifier:
          UpdateControlText(mThresholdT, mOldThresholdTxt, false);
@@ -820,10 +845,10 @@ void EffectDistortion::UpdateUI()
 
          UpdateControl(ID_Threshold, false, defaultLabel(0));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Distortion amount"));
+         UpdateControl(ID_Param1, true, XO("Distortion amount"));
          UpdateControl(ID_Param2, false, defaultLabel(3));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, true, wxEmptyString);
+         UpdateControl(ID_DCBlock, true, {});
          break;
       case kHardLimiter:
          UpdateControlText(mThresholdT, mOldThresholdTxt, true);
@@ -832,12 +857,12 @@ void EffectDistortion::UpdateUI()
          UpdateControlText(mParam2T, mOldParam2Txt, true);
          UpdateControlText(mRepeatsT, mOldRepeatsTxt, false);
 
-         UpdateControl(ID_Threshold, true, _("dB Limit"));
+         UpdateControl(ID_Threshold, true, XO("dB Limit"));
          UpdateControl(ID_NoiseFloor, false, defaultLabel(1));
-         UpdateControl(ID_Param1, true, _("Wet level"));
-         UpdateControl(ID_Param2, true, _("Residual level"));
+         UpdateControl(ID_Param1, true, XO("Wet level"));
+         UpdateControl(ID_Param2, true, XO("Residual level"));
          UpdateControl(ID_Repeats, false, defaultLabel(4));
-         UpdateControl(ID_DCBlock, false, wxEmptyString);
+         UpdateControl(ID_DCBlock, false, {});
          break;
       default:
          UpdateControl(ID_Threshold,   true, defaultLabel(0));
@@ -845,75 +870,86 @@ void EffectDistortion::UpdateUI()
          UpdateControl(ID_Param1,      true, defaultLabel(2));
          UpdateControl(ID_Param2,      true, defaultLabel(3));
          UpdateControl(ID_Repeats,     true, defaultLabel(4));
-         UpdateControl(ID_DCBlock,     false, wxEmptyString);
+         UpdateControl(ID_DCBlock,     false, {});
    }
 }
 
-void EffectDistortion::UpdateControl(control id, bool enabled, wxString name)
+void EffectDistortion::UpdateControl(
+   control id, bool enabled, TranslatableString name)
 {
-   wxString suffix = _(" (Not Used):");
+   auto suffix = XO("(Not Used):");
    switch (id)
    {
-      case ID_Threshold:
+      case ID_Threshold: {
          /* i18n-hint: Control range. */
-         if (enabled) suffix = _(" (-100 to 0 dB):");
-         name += suffix;
+         if (enabled) suffix = XO("(-100 to 0 dB):");
+         name.Join( suffix, wxT(" ") );
 
          // Logarithmic slider is set indirectly
          mThreshold = DB_TO_LINEAR(mParams.mThreshold_dB);
          mThresholdS->SetValue((int) (mThreshold * SCL_Threshold_dB + 0.5));
 
-         mThresholdTxt->SetLabel(name);
-         mThresholdS->SetName(name);
-         mThresholdT->SetName(name);
+         auto translated = name.Translation();
+         mThresholdTxt->SetLabel(translated);
+         mThresholdS->SetName(translated);
+         mThresholdT->SetName(translated);
          mThresholdS->Enable(enabled);
          mThresholdT->Enable(enabled);
          break;
-      case ID_NoiseFloor:
+      }
+      case ID_NoiseFloor: {
          /* i18n-hint: Control range. */
-         if (enabled) suffix = _(" (-80 to -20 dB):");
-         name += suffix;
+         if (enabled) suffix = XO("(-80 to -20 dB):");
+         name.Join( suffix, wxT(" ") );
 
-         mNoiseFloorTxt->SetLabel(name);
-         mNoiseFloorS->SetName(name);
-         mNoiseFloorT->SetName(name);
+         auto translated = name.Translation();
+         mNoiseFloorTxt->SetLabel(translated);
+         mNoiseFloorS->SetName(translated);
+         mNoiseFloorT->SetName(translated);
          mNoiseFloorS->Enable(enabled);
          mNoiseFloorT->Enable(enabled);
          break;
-      case ID_Param1:
+      }
+      case ID_Param1: {
          /* i18n-hint: Control range. */
-         if (enabled) suffix = _(" (0 to 100):");
-         name += suffix;
+         if (enabled) suffix = XO("(0 to 100):");
+         name.Join( suffix, wxT(" ") );
 
-         mParam1Txt->SetLabel(name);
-         mParam1S->SetName(name);
-         mParam1T->SetName(name);
+         auto translated = name.Translation();
+         mParam1Txt->SetLabel(translated);
+         mParam1S->SetName(translated);
+         mParam1T->SetName(translated);
          mParam1S->Enable(enabled);
          mParam1T->Enable(enabled);
          break;
-      case ID_Param2:
+      }
+      case ID_Param2: {
          /* i18n-hint: Control range. */
-         if (enabled) suffix = _(" (0 to 100):");
-         name += suffix;
+         if (enabled) suffix = XO("(0 to 100):");
+         name.Join( suffix, wxT(" ") );
 
-         mParam2Txt->SetLabel(name);
-         mParam2S->SetName(name);
-         mParam2T->SetName(name);
+         auto translated = name.Translation();
+         mParam2Txt->SetLabel(translated);
+         mParam2S->SetName(translated);
+         mParam2T->SetName(translated);
          mParam2S->Enable(enabled);
          mParam2T->Enable(enabled);
          break;
-      case ID_Repeats:
+      }
+      case ID_Repeats: {
          /* i18n-hint: Control range. */
-         if (enabled) suffix = _(" (0 to 5):");
-         name += suffix;
+         if (enabled) suffix = XO("(0 to 5):");
+         name.Join( suffix, wxT(" ") );
 
-         mRepeatsTxt->SetLabel(name);
-         mRepeatsS->SetName(name);
-         mRepeatsT->SetName(name);
+         auto translated = name.Translation();
+         mRepeatsTxt->SetLabel(translated);
+         mRepeatsS->SetName(translated);
+         mRepeatsT->SetName(translated);
          mRepeatsS->Enable(enabled);
          mRepeatsT->Enable(enabled);
          break;
-      case ID_DCBlock:
+      }
+      case ID_DCBlock: {
          if (enabled) {
             mDCBlockCheckBox->SetValue(mbSavedFilterState);
             mParams.mDCBlock = mbSavedFilterState;
@@ -925,20 +961,22 @@ void EffectDistortion::UpdateControl(control id, bool enabled, wxString name)
 
          mDCBlockCheckBox->Enable(enabled);
          break;
-      default: break;
+      }
+      default:
+         break;
    }
 }
 
 void EffectDistortion::UpdateControlText(wxTextCtrl* textCtrl, wxString& string, bool enabled)
 {
    if (enabled) {
-      if (textCtrl->GetValue() == wxEmptyString)
+      if (textCtrl->GetValue().empty())
          textCtrl->SetValue(string);
       else
          string = textCtrl->GetValue();
    }
    else {
-      if (textCtrl->GetValue() != wxEmptyString)
+      if (!textCtrl->GetValue().empty())
          string = textCtrl->GetValue();
       textCtrl->SetValue(wxT(""));
    }
@@ -1185,7 +1223,7 @@ void EffectDistortion::Leveller()
    // Here we model that more efficiently by repeated passes over a linear table.
    for (int n = STEPS; n < TABLESIZE; n++) {
       mTable[n] = ((double) (n - STEPS) / (double) STEPS);
-      for (int i = 0; i < numPasses; i++) {
+      for (int j = 0; j < numPasses; j++) {
          // Find the highest index for gain adjustment
          int index = numPoints - 1;
          for (int i = index; i >= 0 && mTable[n] < gainLimits[i]; i--) {
