@@ -22,6 +22,7 @@ the audio, rather than actually finding the clicks.
 
 
 #include "../Audacity.h"
+#include "Repair.h"
 
 #include <math.h>
 
@@ -29,9 +30,14 @@ the audio, rather than actually finding the clicks.
 
 #include "../InterpolateAudio.h"
 #include "../WaveTrack.h"
-#include "../widgets/ErrorDialog.h"
+#include "../widgets/AudacityMessageBox.h"
 
-#include "Repair.h"
+#include "LoadEffects.h"
+
+const ComponentInterfaceSymbol EffectRepair::Symbol
+{ XO("Repair") };
+
+namespace{ BuiltinEffectsModule::Registration< EffectRepair > reg; }
 
 EffectRepair::EffectRepair()
 {
@@ -41,19 +47,19 @@ EffectRepair::~EffectRepair()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectRepair::GetSymbol()
+ComponentInterfaceSymbol EffectRepair::GetSymbol()
 {
-   return REPAIR_PLUGIN_SYMBOL;
+   return Symbol;
 }
 
-wxString EffectRepair::GetDescription()
+TranslatableString EffectRepair::GetDescription()
 {
-   return _("Sets the peak amplitude of a one or more tracks");
+   return XO("Sets the peak amplitude of a one or more tracks");
 }
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
 EffectType EffectRepair::GetType()
 {
@@ -74,10 +80,8 @@ bool EffectRepair::Process()
    this->CopyInputTracks(); // Set up mOutputTracks. //v This may be too much copying for EffectRepair.
    bool bGoodResult = true;
 
-   SelectedTrackListOfKindIterator iter(Track::Wave, mOutputTracks.get());
-   WaveTrack *track = (WaveTrack *) iter.First();
    int count = 0;
-   while (track) {
+   for( auto track : mOutputTracks->Selected< WaveTrack >() ) {
       const
       double trackStart = track->GetStartTime();
       const double repair_t0 = std::max(mT0, trackStart);
@@ -91,7 +95,9 @@ bool EffectRepair::Process()
          const auto repair1 = track->TimeToLongSamples(repair_t1);
          const auto repairLen = repair1 - repair0;
          if (repairLen > 128) {
-            ::Effect::MessageBox(_("The Repair effect is intended to be used on very short sections of damaged audio (up to 128 samples).\n\nZoom in and select a tiny fraction of a second to repair."));
+            ::Effect::MessageBox(
+               XO(
+"The Repair effect is intended to be used on very short sections of damaged audio (up to 128 samples).\n\nZoom in and select a tiny fraction of a second to repair.") );
             bGoodResult = false;
             break;
          }
@@ -108,8 +114,10 @@ bool EffectRepair::Process()
          const auto len = s1 - s0;
 
          if (s0 == repair0 && s1 == repair1) {
-            ::Effect::MessageBox(_("Repair works by using audio data outside the selection region.\n\nPlease select a region that has audio touching at least one side of it.\n\nThe more surrounding audio, the better it performs."));
-   ///            The Repair effect needs some data to go on.\n\nPlease select an area to repair with some audio on at least one side (the more the better)."));
+            ::Effect::MessageBox(
+               XO(
+"Repair works by using audio data outside the selection region.\n\nPlease select a region that has audio touching at least one side of it.\n\nThe more surrounding audio, the better it performs.") );
+   ///            The Repair effect needs some data to go on.\n\nPlease select an area to repair with some audio on at least one side (the more the better).") );
             bGoodResult = false;
             break;
          }
@@ -125,7 +133,6 @@ bool EffectRepair::Process()
          }
       }
 
-      track = (WaveTrack *) iter.Next();
       count++;
    }
 
