@@ -19,9 +19,9 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
-#include "ImportPCM.h"
-#include "../Internat.h"
+#include "../Audacity.h" // for USE_* macros
+
+#include "Import.h"
 #include "../Tags.h"
 
 #include <wx/wx.h>
@@ -36,8 +36,14 @@
 
 #include "sndfile.h"
 
+#include "../WaveClip.h"
+#include "../ShuttleGui.h"
+
 #include "../ondemand/ODManager.h"
 #include "../ondemand/ODComputeSummaryTask.h"
+#include "../blockfile/ODPCMAliasBlockFile.h"
+#include "../prefs/QualityPrefs.h"
+#include "../widgets/ProgressDialog.h"
 
 //If OD is enabled, he minimum number of samples a file has to use it.
 //Otherwise, we use the older PCMAliasBlockFile method since it should be fast enough.
@@ -49,6 +55,7 @@
 
 #include "../FileFormats.h"
 #include "../Prefs.h"
+#include "../ShuttleGui.h"
 #include "../WaveTrack.h"
 #include "ImportPlugin.h"
 
@@ -66,41 +73,41 @@
    }
 #endif
 
-#define DESC _("WAV, AIFF, and other uncompressed types")
+#define DESC XO("WAV, AIFF, and other uncompressed types")
 
 class PCMImportPlugin final : public ImportPlugin
 {
 public:
    PCMImportPlugin()
-   :  ImportPlugin(wxArrayString())
+   :  ImportPlugin(sf_get_all_extensions())
    {
-      mExtensions = sf_get_all_extensions();
    }
 
    ~PCMImportPlugin() { }
 
-   wxString GetPluginStringID() { return wxT("libsndfile"); }
-   wxString GetPluginFormatDescription();
-   std::unique_ptr<ImportFileHandle> Open(const wxString &Filename) override;
+   wxString GetPluginStringID() override { return wxT("libsndfile"); }
+   TranslatableString GetPluginFormatDescription() override;
+   std::unique_ptr<ImportFileHandle> Open(
+      const FilePath &Filename, AudacityProject*) override;
 };
 
 
 class PCMImportFileHandle final : public ImportFileHandle
 {
 public:
-   PCMImportFileHandle(wxString name, SFFile &&file, SF_INFO info);
+   PCMImportFileHandle(const FilePath &name, SFFile &&file, SF_INFO info);
    ~PCMImportFileHandle();
 
-   wxString GetFileDescription() override;
+   TranslatableString GetFileDescription() override;
    ByteCount GetFileUncompressedBytes() override;
    ProgressResult Import(TrackFactory *trackFactory, TrackHolders &outTracks,
               Tags *tags) override;
 
    wxInt32 GetStreamCount() override { return 1; }
 
-   const wxArrayString &GetStreamInfo() override
+   const TranslatableStrings &GetStreamInfo() override
    {
-      static wxArrayString empty;
+      static TranslatableStrings empty;
       return empty;
    }
 
@@ -113,18 +120,13 @@ private:
    sampleFormat          mFormat;
 };
 
-void GetPCMImportPlugin(ImportPluginList & importPluginList,
-                        UnusableImportPluginList & WXUNUSED(unusableImportPluginList))
-{
-   importPluginList.push_back( make_movable<PCMImportPlugin>() );
-}
-
-wxString PCMImportPlugin::GetPluginFormatDescription()
+TranslatableString PCMImportPlugin::GetPluginFormatDescription()
 {
     return DESC;
 }
 
-std::unique_ptr<ImportFileHandle> PCMImportPlugin::Open(const wxString &filename)
+std::unique_ptr<ImportFileHandle> PCMImportPlugin::Open(
+   const FilePath &filename, AudacityProject*)
 {
    SF_INFO info;
    wxFile f;   // will be closed when it goes out of scope
@@ -142,7 +144,7 @@ std::unique_ptr<ImportFileHandle> PCMImportPlugin::Open(const wxString &filename
       // still has this bug.
       // This happens in sf_open_fd, which is the very first point of
       // interaction with libsndfile, so the only workaround is to hardcode
-      // ImportPCM to not handle .mp3.  Of couse, this will still fail for mp3s
+      // ImportPCM to not handle .mp3.  Of course, this will still fail for mp3s
       // that are mislabeled with a .wav or other extension.
       // So, in the future we may want to write a simple parser to detect mp3s here.
       return NULL;
@@ -188,7 +190,11 @@ std::unique_ptr<ImportFileHandle> PCMImportPlugin::Open(const wxString &filename
    return std::make_unique<PCMImportFileHandle>(filename, std::move(file), info);
 }
 
-PCMImportFileHandle::PCMImportFileHandle(wxString name,
+static Importer::RegisteredImportPlugin registered{ "PCM",
+   std::make_unique< PCMImportPlugin >()
+};
+
+PCMImportFileHandle::PCMImportFileHandle(const FilePath &name,
                                          SFFile &&file, SF_INFO info)
 :  ImportFileHandle(name),
    mFile(std::move(file)),
@@ -204,17 +210,77 @@ PCMImportFileHandle::PCMImportFileHandle(wxString name,
    // the quality of the original file.
    //
 
-   mFormat = (sampleFormat)
-      gPrefs->Read(wxT("/SamplingRate/DefaultProjectSampleFormat"), floatSample);
+   mFormat = QualityPrefs::SampleFormatChoice();
 
    if (mFormat != floatSample &&
        sf_subtype_more_than_16_bits(mInfo.format))
       mFormat = floatSample;
 }
 
-wxString PCMImportFileHandle::GetFileDescription()
+TranslatableString PCMImportFileHandle::GetFileDescription()
 {
-   return SFCall<wxString>(sf_header_name, mInfo.format);
+   // Library strings
+   // See the major_formats and subtype_formats tables in command.c in
+   // libsndfile for this list of possibilities
+
+using Unevaluated = decltype(
+   /* major_formats */
+     XO("AIFF (Apple/SGI)")
+   , XO("AU (Sun/NeXT)")
+   , XO("AVR (Audio Visual Research)")
+   , XO("CAF (Apple Core Audio File)")
+   /* i18n-hint: "codec" is short for a "coder-decoder" algorithm */
+   , XO("FLAC (FLAC Lossless Audio Codec)")
+   , XO("HTK (HMM Tool Kit)")
+   , XO("IFF (Amiga IFF/SVX8/SV16)")
+   , XO("MAT4 (GNU Octave 2.0 / Matlab 4.2)")
+   , XO("MAT5 (GNU Octave 2.1 / Matlab 5.0)")
+   , XO("MPC (Akai MPC 2k)")
+   , XO("OGG (OGG Container format)")
+   , XO("PAF (Ensoniq PARIS)")
+   , XO("PVF (Portable Voice Format)")
+   , XO("RAW (header-less)")
+   , XO("RF64 (RIFF 64)")
+   , XO("SD2 (Sound Designer II)")
+   , XO("SDS (Midi Sample Dump Standard)")
+   , XO("SF (Berkeley/IRCAM/CARL)")
+   , XO("VOC (Creative Labs)")
+   , XO("W64 (SoundFoundry WAVE 64)")
+   , XO("WAV (Microsoft)")
+   , XO("WAV (NIST Sphere)")
+   , XO("WAVEX (Microsoft)")
+   , XO("WVE (Psion Series 3)")
+   , XO("XI (FastTracker 2)")
+);
+
+using Unevaluated2 = decltype(
+   /* subtype_formats */
+     XO("Signed 8 bit PCM")
+   , XO("Signed 16 bit PCM")
+   , XO("Signed 24 bit PCM")
+   , XO("Signed 32 bit PCM")
+   , XO("Unsigned 8 bit PCM")
+   , XO("32 bit float")
+   , XO("64 bit float")
+   , XO("U-Law")
+   , XO("A-Law")
+   , XO("IMA ADPCM")
+   , XO("Microsoft ADPCM")
+   , XO("GSM 6.10")
+   , XO("32kbs G721 ADPCM")
+   , XO("24kbs G723 ADPCM")
+   , XO("12 bit DWVW")
+   , XO("16 bit DWVW")
+   , XO("24 bit DWVW")
+   , XO("VOX ADPCM")
+   , XO("16 bit DPCM")
+   , XO("8 bit DPCM")
+   , XO("Vorbis")
+);
+
+   auto untranslated = SFCall<wxString>(sf_header_name, mInfo.format);
+   return TranslatableString{
+      untranslated, {} };
 }
 
 auto PCMImportFileHandle::GetFileUncompressedBytes() -> ByteCount
@@ -222,11 +288,14 @@ auto PCMImportFileHandle::GetFileUncompressedBytes() -> ByteCount
    return mInfo.frames * mInfo.channels * SAMPLE_SIZE(mFormat);
 }
 
+#ifdef EXPERIMENTAL_OD_DATA
 // returns "copy" or "edit" (aliased) as the user selects.
 // if the cancel button is hit then "cancel" is returned.
 static wxString AskCopyOrEdit()
 {
-   wxString oldCopyPref = gPrefs->Read(wxT("/FileFormats/CopyOrEditUncompressedData"), wxT("copy"));
+
+   auto oldCopyPref = FileFormatsCopyOrEditSetting.Read();
+
    bool firstTimeAsk    = gPrefs->Read(wxT("/Warnings/CopyOrEditUncompressedDataFirstAsk"), true)?true:false;
    bool oldAskPref      = gPrefs->Read(wxT("/Warnings/CopyOrEditUncompressedDataAsk"), true)?true:false;
 
@@ -234,7 +303,7 @@ static wxString AskCopyOrEdit()
    // This effectively does a one-time change to the preferences.
    if (firstTimeAsk) {
       if (oldCopyPref != wxT("copy")) {
-         gPrefs->Write(wxT("/FileFormats/CopyOrEditUncompressedData"), wxT("copy"));
+         FileFormatsCopyOrEditSetting.Write( wxT("copy") );
          oldCopyPref = wxT("copy");
       }
       gPrefs->Write(wxT("/Warnings/CopyOrEditUncompressedDataFirstAsk"), (long) false);
@@ -244,24 +313,21 @@ static wxString AskCopyOrEdit()
    // check the current preferences for whether or not we should ask the user about this.
    if (oldAskPref) {
       wxString newCopyPref = wxT("copy");
-      wxDialogWrapper dialog(nullptr, -1, wxString(_("Warning")));
-      dialog.SetName(dialog.GetTitle());
+      wxDialogWrapper dialog( nullptr, -1, XO("Warning") );
+      dialog.SetName();
 
-      wxBoxSizer *vbox;
-      dialog.SetSizer(vbox = safenew wxBoxSizer(wxVERTICAL));
-
-      wxString clause1 = _(
+      auto clause1 = XO(
 "When importing uncompressed audio files you can either copy them into the project,"
 " or read them directly from their current location (without copying).\n\n"
       );
 
-      wxString clause2 = oldCopyPref == wxT("copy")
-         ? _("Your current preference is set to copy in.\n\n")
-         : _("Your current preference is set to read directly.\n\n")
+      auto clause2 = (oldCopyPref == wxT("copy"))
+         ? XO("Your current preference is set to copy in.\n\n")
+         : XO("Your current preference is set to read directly.\n\n")
       ;
 
-      wxString clause3 = _(
-"Reading the files directly allows you to play or edit them almost immediately.  "
+      auto clause3 = XO(
+"Reading the files directly allows you to play or edit them almost immediately. "
 "This is less safe than copying in, because you must retain the files with their "
 "original names in their original locations.\n"
 "Help > Diagnostics > Check Dependencies will show the original names and locations of any files "
@@ -269,36 +335,31 @@ static wxString AskCopyOrEdit()
 "How do you want to import the current file(s)?"
       );
 
-      wxStaticText *message =
-         safenew wxStaticText(&dialog, -1, clause1 + clause2 + clause3);
-
-      message->Wrap(500);
-      message->SetName(message->GetLabel());
-
-      vbox->Add(message, 1, wxALL | wxEXPAND, 10);
-
-      wxStaticBox *box = safenew wxStaticBox(&dialog, -1, _("Choose an import method"));
-      box->SetName(box->GetLabel());
+      ShuttleGui S{ &dialog, eIsCreating };
+      S.SetBorder(10);
+      S
+         .Position( wxALL | wxEXPAND )
+         .AddUnits( clause1 + clause2 + clause3, 500);
 
       wxRadioButton *aliasRadio;
       wxRadioButton *copyRadio;
       wxCheckBox *dontAskNextTimeBox;
 
+      S.StartStatic(XO("Choose an import method"));
       {
-         auto boxsizer = std::make_unique<wxStaticBoxSizer>(box, wxVERTICAL);
+         S.SetBorder(0);
 
-         copyRadio = safenew wxRadioButton(&dialog, -1, _("Make a &copy of the files before editing (safer)"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
-         boxsizer->Add(copyRadio, 0, wxALL);
-         copyRadio->SetName(wxStripMenuCodes(copyRadio->GetLabel()));
+         copyRadio = S.AddRadioButton(
+            XO("Make a &copy of the files before editing (safer)") );
 
-         aliasRadio = safenew wxRadioButton(&dialog, -1, _("Read the files &directly from the original (faster)"));
-         boxsizer->Add(aliasRadio, 0, wxALL);
-         aliasRadio->SetName(wxStripMenuCodes(aliasRadio->GetLabel()));
+         aliasRadio = S.AddRadioButtonToGroup(
+            XO("Read the files &directly from the original (faster)") );
 
-         dontAskNextTimeBox = safenew wxCheckBox(&dialog, -1, _("Don't &warn again and always use my choice above"));
-         boxsizer->Add(dontAskNextTimeBox, 0, wxALL);
-         vbox->Add(boxsizer.release(), 0, wxALL, 10);
+         dontAskNextTimeBox = S.AddCheckBox(
+            XO("Don't &warn again and always use my choice above"),
+            wxT("false"));
       }
+      S.EndStatic();
 
       dontAskNextTimeBox->SetName(wxStripMenuCodes(dontAskNextTimeBox->GetLabel()));
 
@@ -306,8 +367,8 @@ static wxString AskCopyOrEdit()
       wxRadioButton *prefsRadio = oldCopyPref == wxT("copy") ? copyRadio : aliasRadio;
       prefsRadio->SetValue(true);
 
-      wxSizer *buttonSizer = dialog.CreateButtonSizer(wxOK | wxCANCEL);
-      vbox->Add(buttonSizer, 0, wxALL | wxEXPAND, 10);
+      S.SetBorder( 10 );
+      S.AddStandardButtons( eOkButton | eCancelButton );
 
       dialog.SetSize(dialog.GetBestSize());
       dialog.Layout();
@@ -327,13 +388,14 @@ static wxString AskCopyOrEdit()
 
       // if the preference changed, save it.
       if (newCopyPref != oldCopyPref) {
-         gPrefs->Write(wxT("/FileFormats/CopyOrEditUncompressedData"), newCopyPref);
+         FileFormatsCopyOrEditSetting.Write( newCopyPref );
          gPrefs->Flush();
       }
       oldCopyPref = newCopyPref;
    }
    return oldCopyPref;
 }
+#endif
 
 #ifdef USE_LIBID3TAG
 struct id3_tag_deleter {
@@ -350,6 +412,7 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
 
    wxASSERT(mFile.get());
 
+#ifdef EXPERIMENTAL_OD_DATA
    // Get the preference / warn the user about aliased files.
    wxString copyEdit = AskCopyOrEdit();
 
@@ -360,31 +423,18 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
    bool doEdit = false;
    if (copyEdit.IsSameAs(wxT("edit"), false))
       doEdit = true;
+#endif
 
 
    CreateProgress();
 
-   TrackHolders channels(mInfo.channels);
+   NewChannelGroup channels(mInfo.channels);
 
-   auto iter = channels.begin();
-   for (int c = 0; c < mInfo.channels; ++iter, ++c) {
-      *iter = trackFactory->NewWaveTrack(mFormat, mInfo.samplerate);
-
-      if (mInfo.channels > 1)
-         switch (c) {
-         case 0:
-            iter->get()->SetChannel(Track::LeftChannel);
-            break;
-         case 1:
-            iter->get()->SetChannel(Track::RightChannel);
-            break;
-         default:
-            iter->get()->SetChannel(Track::MonoChannel);
-         }
-   }
-
-   if (mInfo.channels == 2) {
-      channels.begin()->get()->SetLinked(true);
+   {
+      // iter not used outside this scope.
+      auto iter = channels.begin();
+      for (int c = 0; c < mInfo.channels; ++iter, ++c)
+         *iter = trackFactory->NewWaveTrack(mFormat, mInfo.samplerate);
    }
 
    auto fileTotalFrames =
@@ -392,6 +442,7 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
    auto maxBlockSize = channels.begin()->get()->GetMaxBlockSize();
    auto updateResult = ProgressResult::Cancelled;
 
+#ifdef EXPERIMENTAL_OD_DATA
    // If the format is not seekable, we must use 'copy' mode,
    // because 'edit' mode depends on the ability to seek to an
    // arbitrary location in the file.
@@ -415,7 +466,18 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
 
          auto iter = channels.begin();
          for (int c = 0; c < mInfo.channels; ++iter, ++c)
-            iter->get()->AppendAlias(mFilename, i, blockLen, c,useOD);
+            iter->get()->RightmostOrNewClip()->AppendBlockFile(
+               [&]( wxFileNameWrapper filePath, size_t len ) {
+                  return useOD
+                     ? make_blockfile<ODPCMAliasBlockFile>(
+                        std::move(filePath), wxFileNameWrapper{ mFilename },
+                        i, len, c)
+                     : make_blockfile<PCMAliasBlockFile>(
+                        std::move(filePath), wxFileNameWrapper{ mFilename },
+                        i, len, c);
+               },
+               blockLen
+            );
 
          if (++updateCounter == 50) {
             updateResult = mProgress->Update(
@@ -436,16 +498,16 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
 
       if(useOD)
       {
-         auto computeTask = make_movable<ODComputeSummaryTask>();
+         auto computeTask = std::make_unique<ODComputeSummaryTask>();
          bool moreThanStereo = mInfo.channels>2;
          for (const auto &channel : channels)
          {
-            computeTask->AddWaveTrack(channel.get());
+            computeTask->AddWaveTrack(channel);
             if(moreThanStereo)
             {
-               //if we have 3 more channels, they get imported on seperate tracks, so we add individual tasks for each.
+               //if we have 3 more channels, they get imported on separate tracks, so we add individual tasks for each.
                ODManager::Instance()->AddNewTask(std::move(computeTask));
-               computeTask = make_movable<ODComputeSummaryTask>();
+               computeTask = std::make_unique<ODComputeSummaryTask>();
             }
          }
          //if we have a linked track, we add ONE task.
@@ -453,7 +515,10 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
             ODManager::Instance()->AddNewTask(std::move(computeTask));
       }
    }
-   else {
+   else 
+#endif
+
+   {
       // Otherwise, we're in the "copy" mode, where we read in the actual
       // samples from the file and store our own local copy of the
       // samples in the tracks.
@@ -529,10 +594,11 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
       return updateResult;
    }
 
-   for(const auto &channel : channels) {
+   for(const auto &channel : channels)
       channel->Flush();
-   }
-   outTracks.swap(channels);
+
+   if (!channels.empty())
+      outTracks.push_back(std::move(channels));
 
    const char *str;
 
@@ -689,8 +755,8 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
                   ustr = id3_field_getstring(&frame->fields[1]);
                   if (ustr) {
                      // Is this duplication really needed?
-                     MallocString<> str{ (char *)id3_ucs4_utf8duplicate(ustr) };
-                     n = UTF8CTOWX(str.get());
+                     MallocString<> convStr{ (char *)id3_ucs4_utf8duplicate(ustr) };
+                     n = UTF8CTOWX(convStr.get());
                   }
 
                   ustr = id3_field_getstring(&frame->fields[2]);
@@ -701,11 +767,11 @@ ProgressResult PCMImportFileHandle::Import(TrackFactory *trackFactory,
 
                if (ustr) {
                   // Is this duplication really needed?
-                  MallocString<> str{ (char *)id3_ucs4_utf8duplicate(ustr) };
-                  v = UTF8CTOWX(str.get());
+                  MallocString<> convStr{ (char *)id3_ucs4_utf8duplicate(ustr) };
+                  v = UTF8CTOWX(convStr.get());
                }
 
-               if (!n.IsEmpty() && !v.IsEmpty()) {
+               if (!n.empty() && !v.empty()) {
                   tags->SetTag(n, v);
                }
             }
