@@ -34,7 +34,7 @@ DECLARE_EXPORTED_EVENT_TYPE(AUDACITY_DLL_API, EVT_FREQUENCYTEXTCTRL_UPDATED, -1)
 DECLARE_EXPORTED_EVENT_TYPE(AUDACITY_DLL_API, EVT_BANDWIDTHTEXTCTRL_UPDATED,
                             -1);
 
-/** \brief struct to hold a formatting control string and it's user facing name
+/** \brief struct to hold a formatting control string and its user facing name
  * Used in an array to hold the built-in time formats that are always available
  * to the user */
 struct BuiltinFormatString;
@@ -49,6 +49,7 @@ public:
 
    enum Type {
       TIME,
+      ATIME, // for Audio time control.
       FREQUENCY,
       BANDWIDTH,
    };
@@ -56,6 +57,7 @@ public:
    static NumericFormatSymbol DefaultSelectionFormat();
    static NumericFormatSymbol TimeAndSampleFormat();
    static NumericFormatSymbol SecondsFormat();
+   static NumericFormatSymbol HoursMinsSecondsFormat();
    static NumericFormatSymbol HundredthsFormat();
    static NumericFormatSymbol HertzFormat();
    
@@ -79,12 +81,17 @@ public:
    virtual void ControlsToValue();
 
 private:
-   void ParseFormatString(const wxString & untranslatedFormat);
+   void ParseFormatString(const TranslatableString & untranslatedFormat);
 
 public:
    void PrintDebugInfo();
-   void SetFormatName(const NumericFormatSymbol & formatName);
-   void SetFormatString(const wxString & formatString);
+
+   // returns true iff the format name really changed:
+   bool SetFormatName(const NumericFormatSymbol & formatName);
+
+   // returns true iff the format string really changed:
+   bool SetFormatString(const TranslatableString & formatString);
+
    void SetSampleRate(double sampleRate);
    void SetValue(double newValue);
    void SetMinValue(double minValue);
@@ -100,8 +107,8 @@ public:
 
    int GetNumBuiltins();
    NumericFormatSymbol GetBuiltinName(const int index);
-   wxString GetBuiltinFormat(const int index);
-   wxString GetBuiltinFormat(const NumericFormatSymbol & name);
+   TranslatableString GetBuiltinFormat(const int index);
+   TranslatableString GetBuiltinFormat(const NumericFormatSymbol & name);
 
    // Adjust the value by the number "steps" in the active format.
    // Increment if "dir" is 1, decrement if "dir" is -1.
@@ -119,7 +126,7 @@ protected:
    double         mMaxValue;
    double         mInvalidValue;
 
-   wxString       mFormatString;
+   TranslatableString mFormatString;
 
    std::vector<NumericField> mFields;
    wxString       mPrefix;
@@ -153,7 +160,7 @@ class NumericTextCtrl final : public wxControl, public NumericConverter
       bool menuEnabled { true };
       bool hasInvalidValue { false };
       double invalidValue { -1.0 };
-      wxString format {};
+      TranslatableString format {};
       bool hasValue { false };
       double value{ -1.0 };
 
@@ -165,7 +172,7 @@ class NumericTextCtrl final : public wxControl, public NumericConverter
       Options &InvalidValue (bool has, double v = -1.0)
          { hasInvalidValue = has, invalidValue = v; return *this; }
       // use a custom format not in the tables:
-      Options &Format (const wxString &f)
+      Options &Format (const TranslatableString &f)
          { format = f; return *this; }
       Options &Value (bool has, double v)
          { hasValue = has, value = v; return *this; }
@@ -182,16 +189,27 @@ class NumericTextCtrl final : public wxControl, public NumericConverter
 
    virtual ~NumericTextCtrl();
 
+   // Hide the inherited function that takes wxString
+   void SetName( const TranslatableString &name );
+
+   wxSize ComputeSizing(bool update = true, wxCoord digitW = 0, wxCoord digitH = 0);
    bool Layout() override;
    void Fit() override;
 
    void SetSampleRate(double sampleRate);
    void SetValue(double newValue);
-   void SetFormatString(const wxString & formatString);
-   void SetFormatName(const NumericFormatSymbol & formatName);
+
+   // returns true iff the format string really changed:
+   bool SetFormatString(const TranslatableString & formatString);
+
+   // returns true iff the format name really changed:
+   bool SetFormatName(const NumericFormatSymbol & formatName);
 
    void SetFieldFocus(int /* digit */);
 
+   wxSize GetDimensions() { return wxSize(mWidth + mButtonWidth, mHeight); }
+   wxSize GetDigitSize() { return wxSize(mDigitBoxW, mDigitBoxH); }
+   void SetDigitSize(int width, int height);
    void SetReadOnly(bool readOnly = true);
    void EnableMenu(bool enable = true);
 
